//===========================================================================
//
// Mesh.h
//
// By Poman So
//
//===========================================================================

#ifndef _Mesh_h_
#define _Mesh_h_

#include <float.h>
#include "Real.h"
#include "Vector.h"
#include "Node.h"


enum Dist{ CONSTANT, HALF_SINE, HALF_COSINE };


#define N_Lines	18

class Mesh: public Node{
//--------------//
//	data	//
//--------------//
protected:
// number of nodes in x, y and z-direction.
	Uint nx,ny,nz;

// dimensions of the mesh
	Real x,y,z;

// pointers to simulation memory, each memory block contains nxyz node;
// the formula to map array index (i,j,k) to memory location is:
// nyz = ny*nz;		address = i*nyz+j*nz+k = Addr(i,j,k);
// note that  (nx-1)*nyz + (ny-1)*nz + (nz-1) == nxyz-1 == max address.
	Uint nyz,nxyz;
	Real *vp[N_Lines];

// number of types of nodes, pointers to node index array and to Node array.
	Uint nNodes;
	char *node_index,current_index;
	Node *nodes;
		
// numbers of boundaries, pointers to the boundary vectors in discrete
// representations, impulse reflection coefficients of the boundaries.
	Uint nb;
	VecU *n1p,*n2p;
	Real *refl;

public:
	Mesh(VecU &vu, VecR &vr, Real er=1, Real ur=1);
		// (1) set nx=vu.x,ny=vu.y,nz=vu.z,x=vr.x,y=vr.y,z=vr.z
		// (2) call Node(x/nx,y/ny,z/nz,er,ur);
		// (3) sets nNodes=0; current_index=0;
		// (4) sets nyz=ny*nz; nxyz=nx*nyz; nb=0;
		// (5) assign NULL to vp[],nodex_index,nodes,n1p,n2p,refl;
	~Mesh();
	int  Addr(int i, int j, int k){return i*nyz+j*nz+k;}
	void ExV(float &v1, float &v2){Real t=v1; v1=v2; v2=t;}
	Real Cal_Dt();		// smallest Cal_Dt() of all the node-types
	void GetMemory();	// allocates simulation memory to mesh
	void Init();		// initialize mesh to zero everywhere
	fstream &IO(fstream &fs);
	void Scatter();		// scatters incident impulses
	void Transfer();	// transfers impulses to neighbouring nodes
	void Reflect(Boolean forward=True);
	void InjectEH(VecU &ijk, VecR &E, VecR &H);
	void InjectEH(VecU &ijk, VecU &IJK, Dist sx, Dist sy, Dist sz,
		      VecR &E, VecR &H);
	void InjectEH(VecR &xyz, VecR &XYZ, Dist sx, Dist sy, Dist sz,
		      VecR &E, VecR &H);
	Real GetField(VecU &ijk, int fieldMode);
	Real GetField(VecR &xyz, int fieldMode);
        void GetField(VecU &p1, VecU &p2, int fieldMode, Real *f,Real mf=1.0);
	void CtoN(VecR &xyz, VecU &ijk);
		// convert xyz from continue cartesian coordinate to discrete
		// node coordinate and store the result in ijk;
	void NtoC(VecU &ijk, VecR &xyz);
		// does the opposite of CtoN;
	void SetNodes(Uint n, Float8 *f8);
	void SetBR(Uint n, Float7 *f7);
		// (1) sets nb=n.
		// (2) assigns memory to n1p,n2p and refl.
		// (3) discretizes f7[0..2] & f7[3..5] and puts the
		//     result in n1p[] & n2p[] respectively.
		// (4) copy f7[][6] to refl[].
};//--- Mesh ---//

#endif

//===========================================================================
//
// End of Mesh.h
//
//===========================================================================
