//////////////////////////////////////////////////////////////////////////////
//
// Author:	Poman So
//		=================================================
//		Department of Electrical and Computer Engineering
//		University of Victoria
//		PO Box 3055, MS 8610
//		Victoria, BC, V8X 3P6, Canada
//		-------------------------------------------------
//		Tel:      250-472-4224
//		Fax:      250-721-6052/6230
//		Internet: Poman.So@ECE.UVic.CA
//		=================================================
//
// File:	Stopwatch.h
//
//////////////////////////////////////////////////////////////////////////////
#ifndef _Stopwatch_h_
#define _Stopwatch_h_

#ifndef _SYS_PARAM_INCLUDED
  #include <sys/param.h> 
#endif


//  On pure BSD systems, the value HZ is missing from sys/param.h.
//  The following makes Stopwatch more portable at the expense of
//  not giving correct results on systems for which HZ is other
//  than 60.


#ifndef HZ
  #define HZ 60
#endif


//////////////////////////////////////////////////////////////////////////////
//
// CLASS TEMPLATE: Stack
//
//////////////////////////////////////////////////////////////////////////////
class Stopwatch
{
public:
  Stopwatch();
  //    Purpose:
  //        Create a new Stopwatch.
  //        The Stopwatch is initially stopped (Running() is 0)
  //        and Real(), System(), and User() all read zero.
  
  int Running()const{ return running; }
  //    Purpose:
  //        Return 0 iff stopwatch is not running.

  void Reset();
  //    Purpose:
  //        Reset the Stopwatch to zero.  It is not necessary 
  //        for the Stopwatch to be stopped in order to reset it.  
  //        If the Stopwatch is running, it will resume running 
  //        (from zero) immediately after resetting.

  void Start();
  //    Purpose:
  //        Start the Stopwatch moving from its present 
  //        reading.  If Running() is initially 0, it
  //        becomes nonzero.  If Running() is initially
  //        non-zero, the call has no effect.

  void Stop();
  //    Purpose:
  //        Stop the Stopwatch.  If Running() is initially
  //        non-zero, it will become zero.  If Running() is
  //        initially zero, the call will have no effect.

  void Update(){ if (running) { Stop(); Start(); } }
  //    Purpose:
  //        Update the Stopwatch.

  double Real()const{ return r_ticks/double(HZ); }
  //    Purpose:
  //        Return the elapsed real time in seconds.

  double System()const{ return s_ticks/double(HZ); }
  //    Purpose:
  //        Return the elapsed system time in seconds.

  double User()const{ return u_ticks/double(HZ); }
  //    Purpose:
  //        Return the elapsed user time in seconds.

  static double Resolution(){ return 1.0/double(HZ); }
  //    Purpose:
  //        Return the resolution of the Stopwatch in seconds.
  
private:
  long old_r_ticks;
  long old_u_ticks;
  long old_s_ticks;
  long r_ticks;
  long u_ticks;
  long s_ticks;
  int  running;
}; 


#endif	// _Stopwatch_h_
//////////////////////////////////////////////////////////////////////////////
//
//	End of Stopwatch.h
//
//////////////////////////////////////////////////////////////////////////////
